@@ -1,33 +1,33 @@
 import ast
-import copy
 import os
 from datetime import datetime as dt
 from typing import Any, Callable, Union
 
-import fs
 import pendulum
 import yaml
 
 from ..config.interface import ConfigInterface
 from ..core.exceptions import ExecutionException
 from ..core.settings import get_settings
-from ..engines import Engine
-from ..utils.dicts import update_dict
+from ..engine import Engine
+from ..storage import Storage
 from ..execution.schedule import Schedule
 from ..experiment.experiment import Experiment
 from ..experiment.parser import parse_experiment
 from ..filesystem import open_fs
+from ..index import Index
 from ..project import Project
 from ..project.export import Export
 from ..registration import Registration
+from ..utils.dicts import update_dict
 from ..utils.formatting import exception_to_str, msg
 from ..utils.host import get_host_info
-from ..utils.traits import Jsonable
 from ..utils.identifiers import (
     decode_experiment_id,
     encode_experiment_id,
     generate_experiment_id,
 )
+from ..utils.traits import Jsonable
 
 
 def to_color(experiment_id):
@@ -42,14 +42,16 @@ class Execution(Jsonable):
         experiment: Union[Experiment, Any],
         storage: Union[dict, str] = None,
         engine: Union[Engine, str, dict, None] = None,
+        index: Union[Index, str, dict, None] = None,
         project: Union[Project, Callable, str, dict] = None,
         seed: Union[int, None, str] = None,
     ):
         self.function = None
 
         self.experiment = None
-        self.storage = None
+        self.storage = Storage()
         self.engine = None
+        self.index = None
         self.project = None
         self.seed = None
 
@@ -73,6 +75,7 @@ class Execution(Jsonable):
         self.set_storage(storage)
         self.set_experiment(experiment)
         self.set_engine(engine)
+        self.set_index(index)
         self.set_seed(seed)
 
     def __call__(self, experiment, storage=None, engine=None, project=None, seed=None):
@@ -114,7 +117,7 @@ class Execution(Jsonable):
                 experiment_directory = dt.now().strftime(experiment_directory)
             except ValueError:
                 pass
-            self.storage["directory"] = experiment_directory.strip("/")
+            self.storage.config["directory"] = experiment_directory.strip("/")
 
         return self
 
@@ -122,14 +125,7 @@ class Execution(Jsonable):
         if storage is None:
             storage = get_settings()["default_storage"]
 
-        if isinstance(storage, dict):
-            storage = copy.deepcopy(storage)
-        elif isinstance(storage, str):
-            storage = {"url": storage}
-        else:
-            storage = {}
-
-        self.storage = storage
+        self.storage = Storage.create(storage)
 
         return self
 
@@ -141,6 +137,14 @@ class Execution(Jsonable):
 
         return self
 
+    def set_index(self, index):
+        if index is None:
+            index = get_settings()["default_index"]
+
+        self.index = Index.get(index)
+
+        return self
+
     def set_project(self, project):
         self.project = Project.create(project)
 
@@ -228,9 +232,14 @@ class Execution(Jsonable):
         return self
 
     def is_submitted(self):
+        # todo: use FileSystem abstraction
+        import fs
+
         try:
             storage = fs.open_fs(
-                os.path.join(self.storage["url"], self.storage.get("directory", "")),
+                os.path.join(
+                    self.storage.config["url"], self.storage.config["directory"]
+                ),
                 create=False,
             )
             return storage.isdir(self.experiment_id)
@@ -246,7 +255,7 @@ class Execution(Jsonable):
         return False
 
     def submit(self):
-        self.storage["experiment"] = self.experiment_id
+        self.storage.config["experiment"] = self.experiment_id
 
         if not self.is_submitted():
             if len(self.schedule) == 0:
@@ -256,32 +265,25 @@ class Execution(Jsonable):
             self.started_at = str(now)
             # do not backup on mem:// filesystem unless explicitly set to True
             code_backup = self.code_backup
-            if code_backup is None and not self.storage.get("url", "mem://").startswith(
+            if code_backup is None and not self.storage.config["url"].startswith(
                 "mem://"
             ):
                 code_backup = True
 
-            storage = self.engine.storage_middleware(self.storage)
+            storage = self.engine.storage_middleware(self.storage.config)
 
-            with open_fs(
-                {
-                    "url": os.path.join(
-                        storage.get("url", "mem://"),
-                        storage.get("directory", ""),
-                        storage["experiment"],
-                    ),
-                    "create": True,
-                }
-            ) as filesystem:
+            url = os.path.join(
+                storage.get("url", "mem://"),
+                storage.get("directory", ""),
+                storage["experiment"],
+            )
+            with open_fs({"url": url, "create": True}) as filesystem:
 
                 if code_backup:
                     self.project.backup_source_code(opener=filesystem.open)
 
                 code_version = self.project.get_code_version()
 
-                filesystem.save_file("execution.json", self.serialize())
-                filesystem.save_file("schedule.json", self.schedule.serialize())
-                filesystem.save_file("host.json", get_host_info())
                 filesystem.save_file(
                     "code.json",
                     {
@@ -298,6 +300,14 @@ class Execution(Jsonable):
                         "code_version": code_version,
                     },
                 )
+                filesystem.save_file("code.diff", self.project.get_diff())
+
+                filesystem.save_file("execution.json", self.serialize())
+                filesystem.save_file("schedule.json", self.schedule.serialize())
+                filesystem.save_file("host.json", get_host_info())
+
+            # todo: pass the data directly as an StorageFileSystemModel
+            self.index.add(url)
 
         return self.engine.submit(self)
 
@@ -465,8 +475,7 @@ class Execution(Jsonable):
             f"\nExecution: {self.experiment_id}\n----------", color="header",
         )
         msg(
-            f"Storage: {os.path.join(self.storage.get('url', 'mem://'), self.storage.get('directory', ''))}",
-            color="blue",
+            f"Storage: {self.storage}", color="blue",
         )
         msg(f"Engine: {repr(self.engine)}", color="blue")
         msg(f"Project: {repr(self.project)}", color="blue")
@@ -515,14 +524,7 @@ class Execution(Jsonable):
 
     def __repr__(self):
         experiment_id = self.experiment_id if isinstance(self.seed, int) else "None"
-        storage = (
-            os.path.join(
-                self.storage.get("url", "-"), self.storage.get("directory", "")
-            )
-            if isinstance(self.storage, dict)
-            else "-"
-        )
-        return f"Execution <{experiment_id}> ({storage})"
+        return f"Execution <{experiment_id}> ({self.storage})"
 
     def __str__(self):
         return self.__repr__()
