@@ -6,11 +6,6 @@ import os
 import pickle
 import sys
 
-import gitignore_parser
-from fs.zipfs import WriteZipFS
-
-from machinable.utils.utils import is_valid_variable_name
-
 from ..config.loader import from_callable as load_config_from_callable
 from ..config.loader import from_file as load_config_file
 from ..config.loader import from_string as load_config_from_string
@@ -22,7 +17,8 @@ from ..core.settings import get_settings
 from ..registration import Registration
 from ..utils.dicts import update_dict
 from ..utils.traits import Jsonable
-from ..utils.vcs import get_commit
+from ..utils.utils import is_valid_variable_name
+from ..utils.vcs import get_commit, get_root_commit, get_diff
 from .manager import fetch_imports
 
 
@@ -241,6 +237,9 @@ class Project(Jsonable):
         # Returns
         True on completion
         """
+        import gitignore_parser
+        from fs.zipfs import WriteZipFS
+
         if opener is None:
             opener = open
 
@@ -306,6 +305,7 @@ class Project(Jsonable):
 
     def get_code_version(self):
         return {
+            "id": get_root_commit(self.config_filepath),
             "project": get_commit(self.config_filepath),
             "vendor": {
                 vendor: get_commit(
@@ -317,6 +317,9 @@ class Project(Jsonable):
             },
         }
 
+    def get_diff(self):
+        return get_diff(self.config_filepath) or ""
+
     def reload_imports(self):
         vendor_caching = self.options["vendor_caching"]
         self.options["vendor_caching"] = False
