@@ -3,8 +3,7 @@ import os
 import platform
 import socket
 import sys
-
-import pkg_resources
+import getpass
 
 from ..registration import Registration
 
@@ -50,6 +49,11 @@ def _python_version():
     return platform.python_version()
 
 
+@register_host_info
+def _user():
+    return getpass.getuser()
+
+
 @register_host_info
 def _environ():
     return os.environ.copy()
@@ -63,6 +67,8 @@ def _argv():
 @register_host_info
 def _machinable_version():
     try:
+        import pkg_resources
+
         return pkg_resources.require("machinable")[0].version
     except:  # noqa: E722
         return "unknown"
